@@ -8,10 +8,11 @@
 
 import all pygwin classes here'''
 
-import inspect
+
+VERSION = '0.1.0'
+
 import pygame as pg
 
-from .colors import Colors
 from .coord import Coord
 from .enum_type import EnumType
 from .draw import Draw
@@ -48,13 +49,12 @@ from .select import Select
 from .item_select import ItemSelect
 from .int_select import IntSelect
 from .text_board import TextBoard
+from .grid import Grid
 from .window import Window
 from .panel import Panel
 from .window_system import WindowSystem
 from .default_style import DefaultStyle
 
-VERSION = '0.0.1'
-
 #  default key bindings:
 #    * escape => close window
 #    * tab => move focus forward
