@@ -2,7 +2,7 @@
 
 '''definition of class ContextStyle'''
 
-from . import Style
+from . import Style, Context
 
 
 class ContextStyle(dict):
@@ -16,19 +16,18 @@ class ContextStyle(dict):
             result = None
         return result
 
-    def set_attr(self, ctx, attr, value, update=True):
+    def set_attr(self, attr, value, ctx=Context.BASE, update=True):
         '''@TODO@'''
         if ctx not in self:
             self[ctx] = Style()
-        if update or attr not in self[ctx]:
+        result = update or attr not in self[ctx]
+        if result:
             self[ctx][attr] = value
-            result = True
-        else:
-            result = False
         return result
 
     def update(self, ctx, style):
         '''@TODO@'''
         if ctx not in self:
-            self[ctx] = Style()
-        self[ctx] = {**self[ctx], **style}
+            self[ctx] = {**style}
+        else:
+            self[ctx] = {**self[ctx], **style}
