@@ -24,6 +24,12 @@ class Image(Node):
         '''the pygame surface of the image'''
         return self.__surface
 
+    def set_surface(self, surface):
+        '''update the pygame surface of the image'''
+        self.__surface = surface
+        self.update_manager()
+        self.reset_size()
+
     def _compute_inner_size(self):
         return self.__surface.get_size()
 
