@@ -9,6 +9,9 @@ class IntSelect(Select):
     '''IntSelect nodes are Select nodes containing integer values in a
     specific range'''
 
+    PREV_LABEL = '-'
+    NEXT_LABEL = '+'
+
     def __init__(self, min_val, max_val, **kwargs):
         '''creates an IntSelect node with values ranging from min_val to
         max_val'''
