@@ -37,12 +37,11 @@ class Keys:
         '''@TODO@'''
         acts = Keys.__MAP.get(key)
         if acts is not None:
+            best = None
             for keys in acts:
-                if all(pressed[key] for key in keys):
-                    ok = True
-                    for i, p in enumerate(pressed):
-                        if p and i != key and i not in keys:
-                            ok = False
-                    if ok:
-                        return acts[keys]
+                if all(pressed[key] for key in keys) \
+                   and (best is None or len(best) < len(keys)):
+                    best = keys
+            if best is not None:
+                return acts[best]
         return None
