@@ -40,17 +40,9 @@ class Node(metaclass=NodeType):  # pylint: disable=R0904,R0902
         self.__link_proc = None
         self.__pos = None
         self.__manager = kwargs.get('manager')
+        self.inherit(cls + list(inspect.getmro(type(self))[:-1]))
         self.set_link(kwargs.get('link'))
 
-        #  make the node inherit from the styles of its style-classes
-        for c in cls + list(inspect.getmro(type(self))[:-1]):
-            if StyleClass.exists(c):
-                style = StyleClass.get(c)
-                for ctx in style:
-                    for attr in style[ctx]:
-                        self.set_style(
-                            attr, style[ctx][attr], ctx=ctx, update=False)
-
     @property
     def parent(self):
         '''@TODO@'''
@@ -71,11 +63,6 @@ class Node(metaclass=NodeType):  # pylint: disable=R0904,R0902
         '''@TODO@'''
         return self.__container_size
 
-    @property
-    def size(self):
-        '''@TODO@'''
-        return self.__size
-
     @property
     def width(self):
         '''@TODO@'''
@@ -92,12 +79,25 @@ class Node(metaclass=NodeType):  # pylint: disable=R0904,R0902
 
     @property
     def pos(self):
-        '''@TODO@'''
+        '''position of the node'''
         return self.__pos
 
+    @property
+    def inner_pos(self):
+        '''position of the node content (i.e., position of the node + padding
+        + border)'''
+        if self.pos is None:
+            return None
+        return Coord.sum(self.__pos, self._inner_shift())
+
+    @property
+    def size(self):
+        '''size of the node (border + padding included)'''
+        return self.__size
+
     @property
     def inner_size(self):
-        '''@TODO@'''
+        '''size of Node minus its padding and border'''
         if self.size is None:
             return None
         w, h = self.size
@@ -106,14 +106,6 @@ class Node(metaclass=NodeType):  # pylint: disable=R0904,R0902
                   None if h is None else h - hdiff)
         return result
 
-    @property
-    def inner_pos(self):
-        '''@TODO@'''
-        if self.pos is None:
-            return None
-        wdiff, hdiff = self._inner_diff()
-        return Coord.sum(self.__pos, (int(wdiff / 2), int(hdiff / 2)))
-
     @property
     def depth(self):
         '''@TODO@'''
@@ -160,8 +152,15 @@ class Node(metaclass=NodeType):  # pylint: disable=R0904,R0902
             return Media.get_image(self.get_style('image-background'))
         return None
 
-    @container_size.setter
-    def container_size(self, size):
+    def set_parent(self, parent):
+        '''@TODO@'''
+        self.__parent = parent
+
+    def set_depth(self, depth):
+        '''@TODO@'''
+        self.__depth = depth
+
+    def set_container_size(self, size):
         '''@TODO@'''
         if size != self.__container_size:
             self.__container_size = size
@@ -170,18 +169,15 @@ class Node(metaclass=NodeType):  # pylint: disable=R0904,R0902
                     node.reset_size()
                 self.compute_size()
 
-    @parent.setter
-    def parent(self, parent):
-        '''@TODO@'''
-        self.__parent = parent
-
-    @depth.setter
-    def depth(self, depth):
-        '''@TODO@'''
-        self.__depth = depth
+    def inherit(self, classes):
+        '''make the node inherit from the styles of classes'''
+        for c in filter(StyleClass.exists, classes):
+            style_class = StyleClass.get(c)
+            for ctx, attr, value in style_class.get_attrs(self):
+                self.set_style(attr, value, ctx=ctx, update=False)
 
     def set_link(self, link):
-        '''@TODO@'''
+        '''update node so that when it is clicked, link() -> bool is called'''
         self.__link = link
         if self.__link_proc is not None:
             self.del_processor(Event.ON_CLICKUP, self.__link_proc)
@@ -192,6 +188,7 @@ class Node(metaclass=NodeType):  # pylint: disable=R0904,R0902
                 return False
             self.add_processor(Event.ON_CLICKUP, event)
             self.__link_proc = event
+            self.inherit(['link'])
 
     def reset_size(self):
         '''@TODO@'''
@@ -203,10 +200,6 @@ class Node(metaclass=NodeType):  # pylint: disable=R0904,R0902
         '''@TODO@'''
         return Coord.rect(self.pos, self.size)
 
-    def get_window_rect(self):
-        '''@TODO@'''
-        return Coord.rect(self.window_pos, self.size)
-
     def get_absolute_rect(self):
         '''@TODO@'''
         return Coord.rect(self.absolute_pos, self.size)
@@ -275,21 +268,21 @@ class Node(metaclass=NodeType):  # pylint: disable=R0904,R0902
 
     def add_child(self, node):
         '''@TODO@'''
-        node.parent = self
+        node.set_parent(self)
         if self.manager is not None:
-            self.manager.updated = True
+            self.manager.set_updated(True)
         for child in node.iter_tree():
             child.set_manager(self.manager)
 
-    def set_bubble(self, bubble, candidate_pos=None):
+    def set_tooltip(self, tooltip, candidate_pos=None):
         '''@TODO@'''
         def pop(_):
             self.window.pop_node(
-                bubble, node=self, candidate_pos=candidate_pos)
+                tooltip, node=self, candidate_pos=candidate_pos)
             return True
 
         def clear(_):
-            if self.window.popped == bubble:
+            if self.window.popped == tooltip:
                 self.window.clear_popped()
                 return True
             return False
@@ -331,7 +324,7 @@ class Node(metaclass=NodeType):  # pylint: disable=R0904,R0902
         '''@TODO@'''
         for manager in [self.root_manager, self.manager]:
             if manager is not None:
-                manager.updated = True
+                manager.set_updated(True)
 
     def compute_size(self):
         '''@TODO@'''
@@ -348,7 +341,7 @@ class Node(metaclass=NodeType):  # pylint: disable=R0904,R0902
                 return None
 
             #  size is expressed as a percentage of avail_size
-            m = Node.__PERCENT_EXPR.match(val)
+            m = Node.__PERCENT_EXPR.fullmatch(val)
             if not m:
                 raise ValueError('could not parse size {}'.format(val))
             percent = int(val[:len(val) - 1])
@@ -404,13 +397,16 @@ class Node(metaclass=NodeType):  # pylint: disable=R0904,R0902
             size = self.size
         size = self.size
         pos = self.pos
-        if bg == 'color':
-            Draw.rectangle(
-                surface,
-                self.get_style('color-background'),
-                Coord.rect(pos, size))
-        elif bg == 'image':
+        if bg == 'image':
             surface.blit(self.bg_img, pos)
+        elif bg == 'color':
+            corner = self.get_style('corner')
+            bg_color = self.get_style('color-background')
+            rect = Coord.rect(pos, size)
+            if corner is None or corner == 0:
+                Draw.rectangle(surface, bg_color, rect)
+            else:
+                Draw.rectangle_rounded(surface, bg_color, rect, corner)
         else:
             raise ValueError('undefined background type: {}'.format(bg))
 
@@ -463,16 +459,25 @@ class Node(metaclass=NodeType):  # pylint: disable=R0904,R0902
             pos = Coord.sum((x, y), self.pos)
             surface.blit(img, pos)
 
+    def __draw_border_color(self, surface):
+        color = self.get_style('color-border')
+        corner = self.get_style('corner')
+        width = self.get_style('border-width')
+        rect = Coord.rect(self.pos, self.size)
+        if width > 0:
+            if corner is not None and corner > 0:
+                Draw.rectangle_rounded(
+                    surface, color, rect, corner, width)
+            else:
+                Draw.rectangle(
+                    surface, color, rect, width)
+
     def __draw_border(self, surface):
         border = self.get_style('border')
         if border == 'image':
             self.__draw_border_images(surface)
         elif border == 'color':
-            color = self.get_style('color-border')
-            width = self.get_style('border-width')
-            if width > 0:
-                Draw.rectangle(
-                    surface, color, Coord.rect(self.pos, self.size), width)
+            self.__draw_border_color(surface)
         elif border is not None:
             raise ValueError('invalid border type: {}'.format(border))
 
@@ -499,6 +504,8 @@ class Node(metaclass=NodeType):  # pylint: disable=R0904,R0902
     def disable(self):
         '''@TODO@'''
         self.set_disabled(True)
+        if self.has_focus():
+            self.lose_focus()
         for node in self.iter_tree():
             if node.manager is not None:
                 node.manager.trigger(Event.ON_DISABLE, None, node)
@@ -631,8 +638,8 @@ class Node(metaclass=NodeType):  # pylint: disable=R0904,R0902
                     has_ctx = Context.check(self, ctx)
                     if evt in Context.EVTS_ON[ctx] and has_ctx:
                         if evt == Event.ON_KEY and \
-                           (not self.has_focus() or \
-                            not self.does_process_key(pgevt.unicode)):
+                           (not self.has_focus() or
+                                not self.does_process_key(pgevt.unicode)):
                             return False
                         Media.play_sound(sound)
                 for ctx in Context.PRIORITY:
@@ -651,13 +658,15 @@ class Node(metaclass=NodeType):  # pylint: disable=R0904,R0902
 
         #  change the attribute in the style dictionary.  exit if the
         #  attribute is not updated
-        if not self.style.set_attr(ctx, attr, value, update=update):
+        if not self.style.set_attr(attr, value, ctx=ctx, update=update):
             return
 
         self.update_manager()
         self.reset_size()
 
-        pred = lambda evt: evt not in self.__procs
+        def pred(evt):
+            return evt not in self.__procs
+
         for evt in filter(pred, Context.EVTS_ON[ctx] + Context.EVTS_OFF[ctx]):
             proc = change_style(evt)
             self.add_processor(evt, proc)
@@ -679,6 +688,10 @@ class Node(metaclass=NodeType):  # pylint: disable=R0904,R0902
     def _draw(self, surface):
         pass
 
+    def _inner_shift(self):
+        w, h = self._inner_diff()
+        return int(w / 2), int(h / 2)
+
     def _inner_diff(self):
         padding = self.get_style('padding')
         border = self.get_style('border')
