@@ -13,4 +13,4 @@ class NodeType(type):
         type.__init__(cls, name, bases, dic)
 
         #  create a new style class for this node class
-        StyleClass.new(cls)
+        StyleClass(cls)
