@@ -23,11 +23,11 @@ class Radiobox(Checkbox):
         if self.value:
             self.__group.select(self)
 
-    def activate(self):
+    def _activate(self):
         '''activate checkbox which is equivalent to having the user clicking
         on it'''
         if not self.value:
-            if Checkbox.activate(self):
+            if Checkbox._activate(self):
                 self.__group.select(self)
                 return True
         return False
