@@ -14,23 +14,21 @@ import pygame as pg
 from pygwin import VERSION, WindowSystem, Panel, Label, Window, Animation
 from pygwin import Checkbox, RadioboxGroup, InputText, Table, Rule, Style
 from pygwin import Image, Media, Cursor, Menu, Gauge, TextBoard, Empty
-from pygwin import ItemSelect, IntSelect, Box, Coord, Frame, Colors, Context
-from pygwin import Radiobox, StyleClass, DefaultStyle, Button
+from pygwin import ItemSelect, IntSelect, Box, Coord, Frame, Context
+from pygwin import Radiobox, StyleClass, DefaultStyle, Button, Grid
 
 
+def get_media_file(f):
+    '''get the absolute path of a file in the media directory of pygwin.test'''
+    return os.path.join(MEDIA_DIR, f)
+
+
+FPS = 50
 MEDIA_DIR = pkg_resources.resource_filename('pygwin.test', 'media')
 MONSTERS = {
-    'orc': (os.path.join(MEDIA_DIR, 'orc.png'), '10', '20', '0', '10'),
-    'elf': (os.path.join(MEDIA_DIR, 'elf.png'), '5', '15', '30', '50'),
-    'dragon': (os.path.join(MEDIA_DIR, 'dragon.png'), '70', '100', '50', '200')
-}
-BUBBLE_STYLE = {
-    'border': 'color',
-    'border-width': 4,
-    'padding': 10,
-    'orientation': 'vertical',
-    'background': 'color',
-    'color-background': (20, 20, 100)
+    'orc': (get_media_file('orc.png'), '10', '20', '0', '10'),
+    'elf': (get_media_file('elf.png'), '5', '15', '30', '50'),
+    'dragon': (get_media_file('dragon.png'), '70', '100', '50', '200')
 }
 CTX_MENU_CANDIDATE_POS = [
     (Coord.RELATIVE, ((Coord.RIGHT, 0), (Coord.BOTTOM, 0))),
@@ -38,28 +36,13 @@ CTX_MENU_CANDIDATE_POS = [
     (Coord.RELATIVE, ((Coord.LEFT, 0), (Coord.BOTTOM, 0))),
     (Coord.RELATIVE, ((Coord.LEFT, 0), (Coord.TOP, 0)))
 ]
-BUTTON_STYLE = {
-    'base': {
-        'background': 'color',
-        'border': 'color',
-        'border-width': 2,
-        'size': (200, 50),
-        'color-background': Colors.BLUE_DARK,
-        'color-border': Colors.BLUE_DARK
-    },
-    'overed': {
-        'color-border': Colors.BLUE_STEEL,
-        'color': Colors.BLUE_STEEL
-    },
-    'focus': {
-        'color-border': Colors.GREEN_YELLOW,
-        'color': Colors.GREEN_YELLOW
-    }
-}
-LBL_LNK_STYLE = {
-    'overed': {
-        'color': Colors.BLUE_STEEL
-    }
+TOOLTIP_STYLE = {
+    'border': 'color',
+    'border-width': 4,
+    'padding': 10,
+    'orientation': 'vertical',
+    'background': 'color',
+    'color-background': (20, 20, 100)
 }
 
 
@@ -125,38 +108,48 @@ def controls_window(win_sys, title):
                      1: Label(str(ages.value))})
         Window(win_sys, tbl, title='results').open()
         return True
-    center = {'halign': 'center'}
+
+    def disable():
+        if btn_disable.node.text == 'disable controls':
+            btn_disable.node.set_text('enable controls')
+            tbl.disable()
+        else:
+            btn_disable.node.set_text('disable controls')
+            tbl.enable()
+        btn_disable.enable()
+        return True
     tbl = Table()
-    ips = list()
-    ips.append(InputText(place_holder='An input text'))
-    ips.append(InputText(place_holder='A larger input text',
-                         style={'size': (300, None)}))
-    ips.append(InputText(place_holder='An input text occupying all space',
-                         style={'size': ('100%', None)}))
-    ips.append(InputText(place_holder='An input text with a prompt',
-                         prompt='prompt> ',
-                         style={'size': ('100%', None)}))
-    ips.append(InputText(place_holder='4-digit',
-                         allowed=string.digits,
-                         max_size=4,
-                         style={'size': (60, None)}))
+    ips = [
+        InputText(place_holder='An input text'),
+        InputText(place_holder='A larger input text',
+                  style={'size': (300, None)}),
+        InputText(place_holder='An input text occupying all space',
+                  style={'size': ('100%', None)}),
+        InputText(place_holder='An input text with a prompt',
+                  prompt='prompt> ',
+                  style={'size': ('100%', None)}),
+        InputText(place_holder='4-digit',
+                  allowed=string.digits,
+                  max_size=4,
+                  style={'size': (60, None)})
+    ]
     cb = Checkbox(value=True)
     grp = RadioboxGroup()
     fruits = ItemSelect({fruit: fruit for fruit in [
         'apple', 'banana', 'chocolate', 'pear']})
     ages = IntSelect(0, 100, init=20, steps=[1, 5])
-    check = Button('check data', link=check, style=center)
+    btn_check = Button('check data', link=check)
+    btn_disable = Button('disable controls', link=disable)
     tbl.new_row({0: Label('to navigate between controls: TAB or SHIFT+TAB')},
                 colspan={0: 2})
     for ip in ips:
         tbl.new_row({0: ip}, colspan={0: 2})
-    tbl.new_row({0: cb, 1: Label('a checkbox', label_for=cb, cls='lbl_lnk')})
+    tbl.new_row({0: cb, 1: Label('a checkbox', label_for=cb)})
     tbl.new_row({0: Label('how are you?')},
                 colspan={0: 2})
     for how in ['fine', 'ok', 'not great']:
-        how_cb = Radiobox(grp, how)
-        tbl.new_row({0: how_cb, 1:
-                     Label(how, label_for=how_cb, cls='lbl_lnk')})
+        cb = Radiobox(grp, how)
+        tbl.new_row({0: cb, 1: Label(how, label_for=cb)})
     tbl.new_row({0: Label('select a fruit (mousewheel can be used)')},
                 colspan={0: 2})
     tbl.new_row({0: fruits},
@@ -165,7 +158,8 @@ def controls_window(win_sys, title):
                 colspan={0: 2})
     tbl.new_row({0: ages},
                 colspan={0: 2})
-    tbl.new_row({0: check},
+    tbl.new_row({0: Box(btn_disable, btn_check,
+                        style={'orientation': 'horizontal'})},
                 colspan={0: 2})
     return Window(win_sys, tbl, title=title)
 
@@ -175,6 +169,7 @@ def fonts_window(win_sys, title):
     def change_name(_):
         lbl.set_style('font', name_select.value)
         return True
+
     def change_size(_):
         lbl.set_style('font-size', size_select.value)
         return True
@@ -183,7 +178,7 @@ def fonts_window(win_sys, title):
     select[Style.DEFAULT['font']] = 'default'
     for f in os.listdir(MEDIA_DIR):
         name, ext = os.path.splitext(f)
-        abs_path = os.path.abspath(os.path.join(MEDIA_DIR, f))
+        abs_path = os.path.abspath(get_media_file(f))
         if ext == '.ttf':
             select[abs_path] = name
     tbl = Table(style={'halign': 'center'})
@@ -199,9 +194,9 @@ def fonts_window(win_sys, title):
 
 def sounds_window(win_sys, title):
     '''sounds window'''
-    click_check_box = os.path.join(MEDIA_DIR, 'click_checkbox.wav')
-    key_input_text = os.path.join(MEDIA_DIR, 'key_inputtext.wav')
-    over_link = os.path.join(MEDIA_DIR, 'over_link.wav')
+    click_check_box = get_media_file('click_checkbox.wav')
+    key_input_text = get_media_file('key_inputtext.wav')
+    over_link = get_media_file('over_link.wav')
 
     tbl = Table()
     lbl = Label('label')
@@ -242,23 +237,20 @@ def cursor_window(win_sys, title):
     def close():
         Cursor.deactivate()
         Window.close(result)
-    cursor_base_img = os.path.join(MEDIA_DIR, 'cursor-base.png')
-    cursor_clicked_img = os.path.join(MEDIA_DIR, 'cursor-clicked.png')
-    cursor_overed_img = os.path.join(MEDIA_DIR, 'cursor-overed.png')
-    Cursor.set_default(cursor_base_img, ctx=Context.BASE)
+    file_elf = get_media_file('elf.png')
+    file_cursor_base = get_media_file('cursor-base.png')
+    file_cursor_clicked = get_media_file('cursor-clicked.png')
+    file_cursor_overed = get_media_file('cursor-overed.png')
+    Cursor.set_default(file_cursor_base, ctx=Context.BASE)
     Cursor.activate()
 
-    elf = Image(Media.get_image(os.path.join(MEDIA_DIR, 'elf.png')))
-    elf.set_style('cursor', cursor_overed_img, ctx='overed')
-    elf.set_style('cursor', cursor_clicked_img, ctx='clicked')
-    elf.set_style('cursor', cursor_clicked_img, ctx='clicked')
-    tbl = Table()
-    tbl.new_row(
-        {0: Label('move over buttons/images or click to change the cursor')})
-    tbl.new_row({0: elf})
-    tbl.new_row({0: Button('button 1')})
-    tbl.new_row({0: Button('button 2')})
-    result = Window(win_sys, tbl, title=title)
+    elf = Image(Media.get_image(file_elf, scale=(128, 128)))
+    elf.set_style('cursor', file_cursor_overed, ctx='overed')
+    elf.set_style('cursor', file_cursor_clicked, ctx='clicked')
+    box = Box(
+        Label('move over/click the image to change the cursor'),
+        elf)
+    result = Window(win_sys, box, title=title)
     result.close = close
     return result
 
@@ -274,17 +266,17 @@ def menus_window(win_sys, title):
     return result
 
 
-def bubbles_window(win_sys, title):
-    '''bubbles window'''
+def tooltips_window(win_sys, title):
+    '''tooltips window'''
     def new_label(lbl, pos):
         result = Label(lbl, style={'halign': 'center'})
-        result.set_bubble(bubble, candidate_pos=[pos])
+        result.set_tooltip(tooltip, candidate_pos=[pos])
         return result
-    bubble = Box(style=BUBBLE_STYLE)
-    bubble.pack(Label('this is the bubble'))
-    bubble.pack(Image(Media.get_image(os.path.join(MEDIA_DIR, 'dragon.png'),
+    tooltip = Box(style=TOOLTIP_STYLE)
+    tooltip.pack(Label('this is the tooltip'))
+    tooltip.pack(Image(Media.get_image(get_media_file('dragon.png'),
                                       scale=(64, 64))))
-    bubble.pack(Image(Media.get_image(os.path.join(MEDIA_DIR, 'elf.png'),
+    tooltip.pack(Image(Media.get_image(get_media_file('elf.png'),
                                       scale=(64, 64))))
     vpos = [
         ('top', Coord.TOP),
@@ -302,7 +294,7 @@ def bubbles_window(win_sys, title):
     ]
 
     tbl = Table(style={'halign': 'center'})
-    tbl.new_row({0: Label('bubbles can pop when overing an item')},
+    tbl.new_row({0: Label('tooltips can pop when overing an item')},
                 colspan={0: 3})
     for clbl, ct in coord_sys:
         tbl.new_row({0: Label('{} positioning'.format(clbl),
@@ -336,7 +328,7 @@ def ctx_menus_window(win_sys, title):
     monsters = list()
     for m in MONSTERS:
         monsters.append(m)
-        menu = Box(style=BUBBLE_STYLE)
+        menu = Box(style=TOOLTIP_STYLE)
         menu.pack(Label(
             'menu',
             style={'halign': 'center'}))
@@ -357,7 +349,7 @@ def ctx_menus_window(win_sys, title):
 def maximised_window(win_sys, title):
     '''maximised window'''
     board = TextBoard(style={'size': ('100%', None)})
-    with open(os.path.join(MEDIA_DIR, 'lorem_ipsum.txt')) as f:
+    with open(get_media_file('lorem_ipsum.txt')) as f:
         text = f.read()
     for paragraph in text.split('\n\n'):
         board.push_text(paragraph)
@@ -380,7 +372,7 @@ def maximised_menu(win_sys, title):
                       style={'expand': True, 'size': ('100%', '100%')})
         items[Image(Media.get_image(MONSTERS[m][0], scale=(64, 64)))] = frame
     for m in MONSTERS:
-        with open(os.path.join(MEDIA_DIR, 'lorem_ipsum.txt')) as f:
+        with open(get_media_file('lorem_ipsum.txt')) as f:
             text = f.read()
         for paragraph in text.split('\n\n'):
             boards[m].push_text(paragraph)
@@ -448,6 +440,7 @@ def absolute_positioning_window(win_sys, title):
                 direction = (-direction[0], direction[1])
             result.append(direction)
         return result
+
     def start_or_stop():
         if btn.node.text == 'stop':
             anim.pause()
@@ -505,13 +498,12 @@ def user_controls_window(win_sys, title):
         StyleClass.rem('img-button')
         StyleClass.rem('input-text')
         Window.close(result)
-        DefaultStyle.set_default()
+        DefaultStyle.load()
+
     def get_imgs(prefix):
         return tuple(map(imgs.get, filter(
             lambda img: img.startswith(prefix + '-'), imgs)))
 
-    DefaultStyle.reset()
-
     ctxs = ['base', 'overed', 'disabled']
     imgs = [
         *['button-' + ctx for ctx in ctxs + ['clicked']],
@@ -539,29 +531,33 @@ def user_controls_window(win_sys, title):
         'input-text'
     ]
     imgs = {
-        name: os.path.join(MEDIA_DIR, name + '.png') for name in imgs
+        name: get_media_file(name + '.png') for name in imgs
     }
 
     #  class for rules
-    cls = StyleClass.get(Rule)
-    cls.set_attr('base', 'image-hrule', get_imgs('hrule'))
-    cls.set_attr('base', 'image-vrule', get_imgs('vrule'))
-    cls.set_attr('base', 'orientation', 'horizontal')
+    StyleClass.get(Rule).set_style({
+        'base': {
+            'image-hrule': get_imgs('hrule'),
+            'image-vrule': get_imgs('vrule'),
+            'orientation': 'horizontal'
+        }
+    })
 
     #  class for windows
-    StyleClass.new('window', {
+    StyleClass('window').set_style({
         'base': {
             'padding': 10,
             'border': 'image',
             'image-border': get_imgs('window-border'),
             'background': 'color',
+            'corner': 0,
             'color-background': (10, 10, 0),
             'image-window-cross': imgs['window-cross']
         }
     })
 
     #  class for frames
-    StyleClass.new('frame', {
+    StyleClass('frame').set_style({
         'base': {
             'padding': 10,
             'image-frame-vscroll-container':
@@ -578,7 +574,7 @@ def user_controls_window(win_sys, title):
     })
 
     #  class for selects
-    StyleClass.new('item-select', {
+    StyleClass('item-select').set_style({
         ctx: {
             'border': None,
             'padding': 0,
@@ -588,7 +584,7 @@ def user_controls_window(win_sys, title):
     })
 
     #  class for int-selects
-    StyleClass.new('int-select', {
+    StyleClass('int-select').set_style({
         ctx: {
             'border': None,
             'padding': 0,
@@ -598,7 +594,7 @@ def user_controls_window(win_sys, title):
     })
 
     #  class for checkboxes
-    StyleClass.new('checkbox', {
+    StyleClass('checkbox').set_style({
         ctx: {
             'border': None,
             'padding': 0,
@@ -608,7 +604,7 @@ def user_controls_window(win_sys, title):
     })
 
     #  class for radioboxes
-    StyleClass.new('radiobox', {
+    StyleClass('radiobox').set_style({
         ctx: {
             'border': None,
             'padding': 0,
@@ -618,7 +614,7 @@ def user_controls_window(win_sys, title):
     })
 
     #  class for buttons
-    StyleClass.new('img-button', {
+    StyleClass('img-button').set_style({
         ctx: {
             'border': None,
             'padding': 0,
@@ -628,7 +624,7 @@ def user_controls_window(win_sys, title):
     })
 
     #  class for input texts
-    StyleClass.new('input-text', {
+    StyleClass('input-text').set_style({
         'base': {
             'border': None,
             'background': 'image',
@@ -691,7 +687,6 @@ def user_controls_window(win_sys, title):
     ctrl = Button('start', cls=['img-button'], style={'halign': 'center'})
     tbl.new_row({0: ctrl}, colspan={0: 2})
 
-    #tbl = Table()
     result = Window(
         win_sys, tbl,
         title=title,
@@ -700,6 +695,45 @@ def user_controls_window(win_sys, title):
     return result
 
 
+def grids_window(win_sys, title):
+    '''grids window'''
+    def add(img):
+        def fun():
+            def link():
+                new.set_selected(not new.is_selected())
+                return True
+            new = Image(img.surface, link=link)
+            grid.insert(0, new)
+            return True
+        return fun
+
+    def delete():
+        to_del = [img for img in grid if img.is_selected()]
+        for node in to_del:
+            grid.remove_node(node)
+        return True
+    size = 64, 64
+    imgs = [
+        Image(Media.get_image(get_media_file(f), scale=size))
+        for f in ['elf.png', 'orc.png', 'dragon.png']
+    ]
+    for img in imgs:
+        img.set_link(add(img))
+    grid = Grid(style={'size': ('100%', None)})
+    board = TextBoard(style={'size': ('100%', None)})
+    board.push_text('''Click on an image to add it in the frame.  You can then
+select images in the frame and click on delete to remove them from the frame.
+The box in the frame will not hold more than 5 images per row.  Deletion will
+automatically rearrange the box.''')
+    box_main = Box(
+        board,
+        Box(*imgs,
+            Label('delete', link=delete, style={'valign': 'center'}),
+            style={'orientation': 'horizontal', 'halign': 'center'}),
+        Frame(grid, style={'size': (400, 400)}))
+    return Window(win_sys, box_main, title=title)
+
+
 def go():
     '''launch the test'''
 
@@ -717,7 +751,7 @@ def go():
             return lambda: fun(win_sys, title).open()
 
         def quit_test():
-            win_sys.closed = True
+            win_sys.set_closed(True)
         tbl = Table()
         buttons = [
             ('basic window', basic_window),
@@ -728,14 +762,15 @@ def go():
             ('cursor', cursor_window),
             ('frames', frames_window),
             ('menus', menus_window),
-            ('bubbles', bubbles_window),
+            ('tooltips', tooltips_window),
             ('text boards', text_boards_window),
             ('maximised window', maximised_window),
             ('maximised menu', maximised_menu),
             ('contextual menus', ctx_menus_window),
             ('animations', animations_window),
             ('absolute positioning', absolute_positioning_window),
-            ('user controls', user_controls_window)
+            ('user controls', user_controls_window),
+            ('grids', grids_window)
         ]
         row = dict()
         for title, fun in buttons:
@@ -756,14 +791,25 @@ def go():
     pg.font.init()
     screen = pg.display.set_mode((1200, 800))
     pg.display.set_caption('pygwin v{} - test application'.format(VERSION))
-    DefaultStyle.set_default()
-    StyleClass.new('button', BUTTON_STYLE)
-    StyleClass.new('lbl_lnk', LBL_LNK_STYLE)
+    DefaultStyle.load()
     win_sys = WindowSystem(screen)
     p = top_panel()
     p.open(pos=(0, 0))
     pg.key.set_repeat(100, 50)
-    win_sys.loop(40)
+
+    clock = pg.time.Clock()
+    win_sys.draw(update=True)
+
+    #  main loop
+    while not win_sys.closed:
+        win_sys.run_animations()
+        for pgevt in pg.event.get():
+            if pgevt.type == pg.QUIT:
+                win_sys.set_closed(True)
+            win_sys.process_pg_event(pgevt)
+        win_sys.draw(update=True)
+        clock.tick(FPS)
+
     pg.quit()
     sys.exit(0)
 
